/**
 * @name laravel-elixir-jsdoc
 * @author Shane Hendrix
 *
 * @desc Generate documentation for javascript files
 *
 * {@link https://github.com/jsdoc3/jsdoc link}
 *
 * @requires jsdoc
 */

/*global require */
'use strict';

var path = require('path');
var gulp   = require('gulp'),
    Elixir = require('laravel-elixir');

var notify = new Elixir.Notification();

// REQUIRE:     File system functions
const fs = require('fs');

/*
 |----------------------------------------------------------------
 | JSDOC
 |----------------------------------------------------------------
 |
 | This task triggers the JSDOC software and shows
 | notifications depending on the success or the
 | failure of the JSDOC via elixir software
 |
 */


/**
 * @name    makeJsDocCommand
 * @desc    Generate command to run through Elixir
 *
 * @param {string} src      Path to source files
 * @returns {string}        Command that will be run through CLI
 */
function makeJsDocCommand (src) {
    let bin        = path.normalize('node_modules/.bin/jsdoc'),
        dest       = './storage/reports/jsdoc',
        argConfig  = '',
        configFile = 'jsdoc.json';

    //noinspection JSUnresolvedVariable,JSUnresolvedFunction
    src = src || Elixir.config.get('assets.js.folder');


    // Auto-detect config file
    try {
        //noinspection JSUnresolvedFunction
        fs.statSync(configFile);

        // Update BIN if detected
        argConfig = `-c ${configFile}`;
    } catch (e) { // eslint-disable-line no-empty
    }

    return bin
        + argConfig
        + ' --recurse ' + src
        + ' --destination ' + dest;
}

//noinspection JSUnresolvedFunction
Elixir.extend('jsdoc', function initJSDOC (src, command) {
    // SET:     Name
    let name = 'JSDOC';

    // SET:    Command
    command = command || makeJsDocCommand(src);

    // SET:     Header display
    //noinspection JSUnresolvedVariable,JSValidateTypes
    Elixir.Log.heading(`Triggering ${name}: ${command}`);

    // FUNCTION:    Error notification
    var onError = function onErrorFunction (err) {
        notify.error(err, `${name} error!`);
        this.emit('end');
    };

    // ADD:     Elixir Task
    //noinspection JSUnresolvedFunction
    new Elixir.Task('jsdoc', function elixirJsDocTask () {  // eslint-disable-line no-new
        //noinspection JSUnresolvedFunction,JSUnresolvedVariable
        return gulp
            .src('')
            .pipe(Elixir.Plugins.shell(command))
            .on('error', onError)
            .pipe(new Elixir.Notification(`${name} completed successfully`));
    });
});
